# coding: utf-8

import logging
import re
import sys
from datetime import datetime

import django.dispatch
import ijson
import django
from django.apps import apps
from django.db import transaction
from django.utils import timezone
from django.utils.text import slugify

from representatives.models import (Country, Mandate, Email, Address, WebSite,
                                    Representative, Constituency, Phone, Group,
                                    Chamber)
from variants import FranceDataVariants


logger = logging.getLogger(__name__)

representative_pre_import = django.dispatch.Signal(
    providing_args=['representative_data'])


def _get_mdef_item(mdef, item, json, default=None):
    if item in mdef:
        try:
            return mdef[item] % json
        except:
            return default

    if '%s_path' % item in mdef:
        return _get_path(json, mdef['%s_path' % item])

    if '%s_fn' % item in mdef:
        return mdef['%s_fn' % item](json)

    return default


def _parse_date(date):
    return datetime.strptime(date, "%Y-%m-%d").date()


def _create_mandate(representative, group, constituency, role='',
                    begin_date=None, end_date=None):
    mandate, _ = Mandate.objects.get_or_create(
        representative=representative,
        group=group,
        constituency=constituency,
        role=role,
        begin_date=begin_date,
        end_date=end_date
    )

    if _:
        logger.debug('Created mandate %s', mandate.pk)


def _get_path(dict_, path):
    '''
    Get value at specific path in dictionary. Path is specified by slash-
    separated string, eg _get_path(foo, 'bar/baz') returns foo['bar']['baz']
    '''
    cur = dict_
    for part in path.split('/'):
        cur = cur[part]
    return cur


class GenericImporter(object):

    def pre_import(self):
        self.import_start_datetime = timezone.now()

    def touch_model(self, model, **data):
        '''
        This method create or look up a model with the given data
        it saves the given model if it exists, updating its
        updated field
        '''

        instance, created = model.objects.get_or_create(**data)

        if not created:
            if instance.updated < self.import_start_datetime:
                instance.save()     # Updates updated field

        return (instance, created)


def ensure_chambers():
    """
    Ensures chambers are created
    """
    france = Country.objects.get(name="France")
    for key in ('AN', 'SEN'):
        variant = FranceDataVariants[key]
        Chamber.objects.get_or_create(name=variant['chamber'],
                                      abbreviation=variant['abbreviation'],
                                      country=france)


class FranceDataImporter(GenericImporter):
    url = 'http://francedata.future/data/parlementaires.json'

    def parse_date(self, date):
        return _parse_date(date)

    def __init__(self, variant):
        self.france = Country.objects.get(name="France")
        self.variant = FranceDataVariants[variant]
        self.chamber = Chamber.objects.get(name=self.variant['chamber'])
        self.ch_constituency, _ = Constituency.objects.get_or_create(
            name=self.variant['chamber'], country=self.france)

    @transaction.atomic
    def manage_rep(self, rep_json):
        '''
        Import a rep as a representative from the json dict fetched from
        FranceData (which comes from nosdeputes.fr)
        '''

        # Some versions of memopol will connect to this and skip inactive reps.
        responses = representative_pre_import.send(sender=self,
                representative_data=rep_json)

        for receiver, response in responses:
            if response is False:
                logger.debug(
                    'Skipping MEP %s', rep_json['nom'])
                return

        changed = False
        slug = slugify('%s-%s' % (
            rep_json['nom'] if 'nom' in rep_json
            else rep_json['prenom'] + " " + rep_json['nom_de_famille'],
            _parse_date(rep_json["date_naissance"])
        ))

        try:
            representative = Representative.objects.get(slug=slug)
        except Representative.DoesNotExist:
            representative = Representative(slug=slug)
            changed = True

        if rep_json['num_circo'] == 'non disponible':
            rep_json['num_circo'] = 'nd'

        # Save representative attributes
        self.import_representative_details(representative, rep_json, changed)

        self.add_mandates(representative, rep_json)

        self.add_contacts(representative, rep_json)

        logger.debug('Imported MEP %s', unicode(representative))

        return representative

    def import_representative_details(self, representative, rep_json, changed):
        active = True
        if rep_json.get("ancien_depute", 0) == 1:
            active = False
        if rep_json.get("ancien_senateur", 0) == 1:
            active = False
        if representative.active != active:
            representative.active = active
            changed = True

        if rep_json.get("date_naissance"):
            birth_date = _parse_date(rep_json["date_naissance"])
            if representative.birth_date != birth_date:
                representative.birth_date = birth_date
                changed = True

        if rep_json.get("lieu_naissance"):
            birth_place = rep_json["lieu_naissance"]
            if representative.birth_place != birth_place:
                representative.birth_place = birth_place
                changed = True

        photo = rep_json['photo_url']
        if representative.photo != photo:
            representative.photo = photo
            changed = True

        first_name = rep_json['prenom']
        if representative.first_name != first_name:
            representative.first_name = first_name
            changed = True

        last_name = rep_json['nom_de_famille']
        if representative.last_name != last_name:
            representative.last_name = last_name
            changed = True

        full_name = rep_json["nom"]
        if representative.full_name != full_name:
            representative.full_name = full_name
            changed = True

        gender_convertion_dict = {u"F": 1, u"H": 2}
        if 'sexe' in rep_json:
            gender = gender_convertion_dict.get(rep_json['sexe'], 0)
        else:
            gender = 0
        if representative.gender != gender:
            representative.gender = gender
            changed = True

        if changed:
            representative.save()

    def add_mandates(self, representative, rep_json):
        '''
        Create mandates from rep data based on variant configuration
        '''

        # Mandate in country group for party constituency
        if rep_json.get('parti_ratt_financier'):
            constituency, _ = Constituency.objects.get_or_create(
                name=rep_json.get('parti_ratt_financier'), country=self.france)

            group, _ = self.touch_model(model=Group,
                                        abbreviation=self.france.code,
                                        kind='country',
                                        name=self.france.name)

            _create_mandate(representative, group, constituency, 'membre')

        # Configurable mandates
        for mdef in self.variant['mandates']:
            if mdef.get('chamber', False):
                chamber = self.chamber
            else:
                chamber = None

            if 'from' in mdef:
                elems = mdef['from'](rep_json)
            else:
                elems = [rep_json]

            for elem in elems:
                name = _get_mdef_item(mdef, 'name', elem, '')
                abbr = _get_mdef_item(mdef, 'abbr', elem, '')

                group, _ = self.touch_model(model=Group,
                                            abbreviation=abbr,
                                            kind=mdef['kind'],
                                            chamber=chamber,
                                            name=name)

                role = _get_mdef_item(mdef, 'role', elem, 'membre')
                start = _get_mdef_item(mdef, 'start', elem, None)
                if start is not None:
                    start = _parse_date(start)
                end = _get_mdef_item(mdef, 'end', elem, None)
                if end is not None:
                    end = _parse_date(end)

                _create_mandate(representative, group, self.ch_constituency,
                                role, start, end)

                logger.debug(
                    '%s => %s: %s of "%s" (%s) %s-%s' % (rep_json['slug'],
                    mdef['kind'], role, name, abbr, start, end))

    def add_contacts(self, representative, rep_json):
        # Chamber page
        changed = False
        try:
            site = WebSite.objects.get(kind=self.variant['abbreviation'],
                                       representative=representative)
        except WebSite.DoesNotExist:
            site = WebSite(kind=self.variant['abbreviation'],
                           representative=representative)
            changed = True

        if site.url != rep_json[self.variant['chamber_url_field']]:
            site.url = rep_json[self.variant['chamber_url_field']]
            changed = True

        if changed:
            site.save()

        # Websites
        websites = rep_json.get('sites_web', [])
        for site in websites:
            if re.search(r'facebook\.com', site['site']):
                self.touch_model(model=WebSite,
                                 url=site['site'],
                                 kind='facebook',
                                 representative=representative
                                 )
            elif not re.search(r'twitter\.com', site['site']):
                self.touch_model(model=WebSite,
                                 url=site['site'],
                                 representative=representative
                                 )

        # Twitter
        if rep_json.get('twitter'):
            tid = rep_json.get('twitter')
            self.touch_model(model=WebSite,
                             representative=representative,
                             kind='twitter',
                             url='http://twitter.com/%s' % tid
                             )

        # E-mails
        emails = rep_json.get('emails', [])
        for email in emails:
            mail = email['email']
            self.touch_model(
                model=Email,
                representative=representative,
                kind=('official' if mail.endswith(self.variant['mail_domain'])
                    else 'other'),
                email=mail)

        # Official address
        off_name = self.variant['off_name']
        official_addr, _ = self.touch_model(model=Address,
                                            representative=representative,
                                            country=self.france,
                                            city=self.variant['off_city'],
                                            street=self.variant['off_street'],
                                            number=self.variant['off_number'],
                                            postcode=self.variant['off_code'],
                                            kind='official',
                                            name=off_name
                                            )

        # Addresses & phone numbers
        addresses = rep_json.get('adresses', [])
        for item in addresses:
            addr = None
            if 'geo' in item:
                props = item['geo'].get('properties', {})
                name = ''

                if item['adresse'].lower().startswith('permanence'):
                    name = 'Permanence'

                addr, _ = self.touch_model(model=Address,
                                           representative=representative,
                                           country=self.france,
                                           city=props.get('city', ''),
                                           street=props.get('street', ''),
                                           number=props.get('housenumber', ''),
                                           postcode=props.get('postcode', ''),
                                           kind='',
                                           name=name
                                           )
            elif item['adresse'].lower().startswith(off_name.lower()):
                addr = official_addr

            if 'tel' in item:
                self.touch_model(model=Phone, address=addr,
                                 representative=representative,
                                 kind='', number=item['tel']
                                 )


def main(stream=None):
    if not apps.ready:
        django.setup()

    ensure_chambers()

    an_importer = FranceDataImporter('AN')
    GenericImporter.pre_import(an_importer)

    sen_importer = FranceDataImporter('SEN')
    GenericImporter.pre_import(sen_importer)

    for data in ijson.items(stream or sys.stdin, ''):
        for rep in data:
            if rep['chambre'] == 'AN':
                an_importer.manage_rep(rep)
            elif rep['chambre'] == 'SEN':
                sen_importer.manage_rep(rep)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def remove_duplicate(apps, schema_editor):
    Country = apps.get_model('representatives', 'country')
    Address = apps.get_model('representatives', 'address')
    if Country.objects.filter(pk=12, code='IL').count():
        Address.objects.filter(country_id=12).update(country_id=1121)
        Country.objects.get(pk=12).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0005_auto_20151212_2251'),
    ]

    operations = [
        migrations.RunPython(remove_duplicate)
    ]



from django import test

from responsediff.response import Response


class RepresentativeManagerTest(test.TestCase):
    fixtures = ['representatives_test.json']

    def functional_test(self, queries, url):
        with self.assertNumQueries(queries):
            result = test.client.Client().get(url)
        Response.for_test(self).assertNoDiff(result)

    def test_constituencies_api(self):
        self.functional_test(1, '/api/constituencies/?format=json')

    def test_groups_api(self):
        self.functional_test(1, '/api/groups/?format=json')

    def test_mandates_api(self):
        self.functional_test(1, '/api/mandates/?format=json')

    def test_chambers_api(self):
        self.functional_test(1, '/api/chambers/?format=json')

    def test_representatives_api(self):
        """
        Queries:

        - representatives,
        - emails,
        - websites,
        - addresses,
        - phones,
        - mandates.
        """
        self.functional_test(6, '/api/representatives/?format=json')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id',
                 models.AutoField(
                     verbose_name='ID',
                     serialize=False,
                     auto_created=True,
                     primary_key=True)),
                ('name',
                 models.CharField(
                     max_length=255)),
                ('code',
                 models.CharField(
                     max_length=2)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

from django.core import serializers
from django.db import migrations

fixture_dir = os.path.abspath(
    os.path.join(
        os.path.dirname(__file__),
        '../fixtures'))
fixture_filename = 'country_initial_data.json'


def load_fixture(apps, schema_editor):
    fixture_file = os.path.join(fixture_dir, fixture_filename)

    fixture = open(fixture_file, 'rb')
    objects = serializers.deserialize('json', fixture, ignorenonexistent=True)
    for obj in objects:
        obj.save()
    fixture.close()


def unload_fixture(apps, schema_editor):
    "Brutally deleting all entries for this model..."

    MyModel = apps.get_model("representatives", "Country")
    MyModel.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(load_fixture, reverse_code=unload_fixture),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0009_order_mandates_by_end_date_descendant'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='mandate',
            options={'ordering': ('-end_date',)},
        ),
        migrations.AlterField(
            model_name='group',
            name='abbreviation',
            field=models.CharField(default=b'', max_length=10, db_index=True, blank=True),
        ),
        migrations.AlterField(
            model_name='group',
            name='kind',
            field=models.CharField(max_length=255, db_index=True),
        ),
    ]



from django.db import models

from rest_framework import (
    filters,
    pagination,
    renderers,
    viewsets,
)

from representatives.serializers import (
    ChamberSerializer,
    ConstituencySerializer,
    GroupSerializer,
    MandateSerializer,
    RepresentativeDetailSerializer,
    RepresentativeSerializer,
)

from .models import (
    Address,
    Chamber,
    Constituency,
    Group,
    Mandate,
    Phone,
    Representative,
)


class DefaultWebPagination(pagination.PageNumberPagination):
    default_web_page_size = 10

    def get_page_size(self, request):
        web = isinstance(request.accepted_renderer,
                         renderers.BrowsableAPIRenderer)
        size = pagination.PageNumberPagination.get_page_size(self, request)

        if web and not size:
            return self.default_web_page_size

        return size


class RepresentativeViewSet(viewsets.ReadOnlyModelViewSet):
    """
    API endpoint that allows representatives to be viewed.
    """
    queryset = Representative.objects.all()
    filter_backends = (
        filters.DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter
    )
    filter_fields = {
        'active': ['exact'],
        'slug': ['exact', 'icontains'],
        'id': ['exact'],
        'first_name': ['exact', 'icontains'],
        'last_name': ['exact', 'icontains'],
        'full_name': ['exact', 'icontains'],
        'gender': ['exact'],
        'birth_place': ['exact'],
        'birth_date': ['exact', 'gte', 'lte'],
    }
    search_fields = ('first_name', 'last_name', 'slug')
    ordering_fields = ('id', 'birth_date', 'last_name', 'full_name')
    pagination_class = DefaultWebPagination

    def get_queryset(self):
        qs = super(RepresentativeViewSet, self).get_queryset()
        qs = qs.prefetch_related(
            'email_set',
            'website_set',
            models.Prefetch(
                'address_set',
                queryset=Address.objects.select_related('country')
            ),
            models.Prefetch(
                'phone_set',
                queryset=Phone.objects.select_related('address__country')
            ),
            'mandates',
        )
        return qs

    def list(self, request):
        self.serializer_class = RepresentativeSerializer
        return super(RepresentativeViewSet, self).list(request)

    def retrieve(self, request, pk=None):
        self.serializer_class = RepresentativeDetailSerializer
        return super(RepresentativeViewSet, self).retrieve(request, pk)


class MandateViewSet(viewsets.ReadOnlyModelViewSet):
    """
    API endpoint that allows mandates to be viewed.
    """
    pagination_class = DefaultWebPagination
    queryset = Mandate.objects.select_related('representative')
    serializer_class = MandateSerializer

    filter_backends = (
        filters.DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter
    )
    filter_fields = {
        'id': ['exact'],
        'group__name': ['exact', 'icontains'],
        'group__abbreviation': ['exact'],
    }
    search_fields = ('group__name', 'group__abbreviation')


class ConstituencyViewSet(viewsets.ReadOnlyModelViewSet):
    pagination_class = DefaultWebPagination
    queryset = Constituency.objects.all()
    serializer_class = ConstituencySerializer


class GroupViewSet(viewsets.ReadOnlyModelViewSet):
    pagination_class = DefaultWebPagination
    queryset = Group.objects.all()
    serializer_class = GroupSerializer


class ChamberViewSet(viewsets.ReadOnlyModelViewSet):
    pagination_class = DefaultWebPagination
    queryset = Chamber.objects.all()
    serializer_class = ChamberSerializer



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0018_auto_20160624_0517'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='chamber',
            name='fingerprint',
        ),
        migrations.RemoveField(
            model_name='constituency',
            name='fingerprint',
        ),
        migrations.RemoveField(
            model_name='group',
            name='fingerprint',
        ),
        migrations.RemoveField(
            model_name='mandate',
            name='fingerprint',
        ),
        migrations.RemoveField(
            model_name='representative',
            name='fingerprint',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0013_constituency_country_related_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Chamber',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('fingerprint', models.CharField(unique=True, max_length=40)),
                ('name', models.CharField(max_length=255)),
                ('country', models.ForeignKey(related_name='chambers', to='representatives.Country', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='group',
            name='chamber',
            field=models.ForeignKey(related_name='groups', to='representatives.Chamber', null=True),
        ),
    ]



# coding: utf-8

from django.db import transaction
from rest_framework import serializers

import representatives.models as models


class CountrySerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Country
        fields = ('name', 'code')


class ChamberSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Chamber
        fields = ('name', 'abbreviation', 'country')


class EmailSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Email
        fields = ('email', 'kind')


class WebsiteSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.WebSite
        fields = ('url', 'kind')

    def validate_url(self, value):
        # Don’t validate url, because it could break import of not proper
        # formed url
        return value


class PhoneSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Phone
        fields = ('number', 'kind')

    def validate_phone(self, value):
        return value


class AddressSerializer(serializers.ModelSerializer):
    country = CountrySerializer()

    class Meta:
        model = models.Address
        fields = ('country', 'city', 'street',
                  'number', 'postcode', 'floor',
                  'office_number', 'kind',
                  )


class ContactField(serializers.Serializer):
    emails = EmailSerializer(many=True)
    phones = PhoneSerializer(many=True)
    websites = WebsiteSerializer(many=True)
    address = AddressSerializer(many=True)

    def get_attribute(self, obj):
        return {
            'emails': obj.email_set.all(),
            'websites': obj.website_set.all(),
            'phones': obj.phone_set.all(),
            'address': obj.address_set.all(),
        }


class ConstituencySerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Constituency
        fields = ('id', 'name')


class GroupSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Group
        fields = ('id', 'name', 'abbreviation', 'kind')


class MandateSerializer(serializers.ModelSerializer):

    # name = serializers.CharField(source='group.name')
    # short_id = serializers.CharField(
    #     source='group.abbreviation', allow_blank=True)
    # kind = serializers.CharField(source='group.kind')
    # constituency = serializers.CharField(source='constituency.name')

    group = serializers.CharField(
        source='group.id',
    )
    constituency = serializers.CharField(
        source='constituency.id'
    )
    representative = serializers.CharField(
        source='representative.id'
    )

    class Meta:
        depth = 1
        model = models.Mandate
        fields = (
            'id',
            'representative',
            'group',
            'constituency',
            'role',
            'begin_date',
            'end_date',
        )

    def to_internal_value(self, data):
        data = super(MandateSerializer, self).to_internal_value(data)
        data['group'] = models.Group.objects.get(
            id=data['group']['id']
        )
        data['constituency'] = models.Constituency.objects.get(
            id=data['constituency']['id']
        )
        data['representative'] = models.Representative.objects.get(
            id=data['representative']['id']
        )
        return data


class MandateDetailSerializer(MandateSerializer):
    group = GroupSerializer()
    constituency = ConstituencySerializer()

    class Meta(MandateSerializer.Meta):
        fields = (
            'id',
            'group',
            'constituency',
            'role',
            'begin_date',
            'end_date',
        )


class RepresentativeSerializer(serializers.ModelSerializer):
    contacts = ContactField()

    class Meta:
        model = models.Representative
        fields = (
            'id',
            'slug',
            'first_name',
            'last_name',
            'full_name',
            'gender',
            'birth_place',
            'birth_date',
            'photo',
            'active',
            'cv',
            'contacts',
            'url',
        )

    @transaction.atomic
    def create(self, validated_data):
        contacts_data = validated_data.pop('contacts')
        representative = models.Representative.objects.create(
            **validated_data
        )
        self._create_contacts(contacts_data, representative)
        return representative

    @transaction.atomic
    def update(self, instance, validated_data):
        contacts_data = validated_data.pop('contacts')
        for attr, value in validated_data.iteritems():
            setattr(instance, attr, value)
        instance.save()
        self._create_contacts(contacts_data, instance)
        return instance

    def touch_model(self, model, **data):
        '''
        This method create or look up a model with the given data
        it saves the given model if it exists, updating its
        updated field
        '''
        instance, created = model.objects.get_or_create(**data)

        if not created:
            instance.save()

        return (instance, created)

    def _create_contacts(self, contacts_data, representative):
        for contact_data in contacts_data['emails']:
            contact_data['representative'] = representative
            self.touch_model(model=models.Email, **contact_data)

        for contact_data in contacts_data['websites']:
            contact_data['representative'] = representative
            self.touch_model(model=models.WebSite, **contact_data)

        for contact_data in contacts_data['address']:
            country, _ = models.Country.objects.get_or_create(
                **contact_data.pop('country')
            )
            phone_set = contact_data.pop('phones')
            contact_data['representative'] = representative
            contact_data['country'] = country
            contact, _ = self.touch_model(model=models.Address, **contact_data)

            for phone_data in phone_set:
                phone_data['representative'] = representative
                phone_data['address'] = contact
                self.touch_model(model=models.Phone, **phone_data)


class RepresentativeDetailSerializer(RepresentativeSerializer):

    mandates = MandateDetailSerializer(many=True)

    class Meta(RepresentativeSerializer.Meta):
        fields = RepresentativeSerializer.Meta.fields + (
            'mandates',
        )



# coding: utf-8

# This file is part of compotista.
#
# compotista is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or any later version.
#
# compotista is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU General Affero Public
# License along with django-representatives.
# If not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2013 Laurent Peuch <cortex@worlddomination.be>
# Copyright (C) 2015 Arnaud Fabre <af@laquadrature.net>

from django.contrib import admin

from .models import (Address, Constituency, Country, Email, Group, Mandate,
                     Phone, Representative, WebSite)


class EmailInline(admin.TabularInline):
    model = Email
    extra = 0


class WebsiteInline(admin.TabularInline):
    model = WebSite
    extra = 0


class AdressInline(admin.StackedInline):
    model = Address
    extra = 0


class PhoneInline(admin.TabularInline):
    model = Phone
    extra = 0


class MandateInline(admin.StackedInline):
    model = Mandate
    extra = 0


class RepresentativeAdmin(admin.ModelAdmin):
    list_display = ('id', 'full_name', 'gender', 'birth_place')
    search_fields = ('first_name', 'last_name', 'birth_place')
    list_filter = ('gender', )
    inlines = [
        PhoneInline,
        EmailInline,
        WebsiteInline,
        AdressInline,
        MandateInline
    ]


class GroupAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'abbreviation', 'kind')
    list_filter = ('kind',)


class MandateAdmin(admin.ModelAdmin):
    list_display = (
        'id',
        'representative',
        'group',
        'role',
        'constituency',
        'begin_date',
        'end_date')
    search_fields = ('representative', 'group', 'constituency')


admin.site.register(Representative, RepresentativeAdmin)
admin.site.register(Country)
admin.site.register(Mandate, MandateAdmin)
admin.site.register(Group, GroupAdmin)
admin.site.register(Constituency)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import hashlib

from django.db import migrations, models
from django.utils.encoding import smart_str


def calculate_hash(obj):
    """
    Computes fingerprint for an object, this code is duplicated from
    representatives.models.HashableModel because we don't have access to model
    methods in a migration scenario.
    """

    hashable_fields = {
        'Chamber': ['name', 'country', 'abbreviation'],
        'Constituency': ['name'],
        'Group': ['name', 'abbreviation', 'kind', 'chamber'],
        'Mandate': ['group', 'constituency', 'role', 'begin_date', 'end_date',
            'representative']
    }

    fingerprint = hashlib.sha1()
    for field_name in hashable_fields[obj.__class__.__name__]:
        field = obj._meta.get_field(field_name)
        if field.is_relation:
            related = getattr(obj, field_name)
            if related is None:
                fingerprint.update(smart_str(related))
            else:
                fingerprint.update(related.fingerprint)
        else:
            fingerprint.update(smart_str(getattr(obj, field_name)))
    obj.fingerprint = fingerprint.hexdigest()
    return obj.fingerprint


def get_or_create(cls, **kwargs):
    """
    Implements get_or_create logic for models that inherit from
    representatives.models.HashableModel because we don't have access to model
    methods in a migration scenario.
    """

    try:
        obj = cls.objects.get(**kwargs)
        created = False
    except cls.DoesNotExist:
        obj = cls(**kwargs)
        created = True
        calculate_hash(obj)
        obj.save()

    return (obj, created)


def migrate_ep_chamber(apps, schema_editor):
    # Get model managers

    Chamber = apps.get_model("representatives", "Chamber")
    Constituency = apps.get_model("representatives", "Constituency")
    Group = apps.get_model("representatives", "Group")
    Mandate = apps.get_model("representatives", "Mandate")

    # Create EP chamber, constituency and group

    name = 'European Parliament'
    abbr = 'EP'

    ep_chamber, _ = get_or_create(Chamber, name=name, abbreviation=abbr)
    ep_constituency, _ = get_or_create(Constituency, name=name)
    ep_group, _ = get_or_create(Group, name=name, kind='chamber',
        abbreviation=abbr, chamber=ep_chamber)

    # Set chamber on groups

    ep_group_kinds = [
        'committee',
        'delegation',
        'group'
    ]

    Group.objects.filter(kind__in=ep_group_kinds).update(chamber=ep_chamber)

    # Set constituency on EP mandates

    ep_mandate_kinds = [
        'committee',
        'delegation',
        'group',
        'organization'
    ]

    Mandate.objects.filter(group__kind__in=ep_mandate_kinds).update(
        constituency=ep_constituency)

    # Create EP mandates with EP constituency & group with same timespan as
    # the country mandate

    for m in Mandate.objects.filter(group__kind__exact='country'):
        mandate, _ = get_or_create(Mandate,
            representative=m.representative,
            group=ep_group,
            constituency=ep_constituency,
            role=m.role,
            begin_date=m.begin_date,
            end_date=m.end_date
        )


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0015_chamber_abbreviation'),
    ]

    operations = [
        migrations.RunPython(migrate_ep_chamber)
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0012_index_group_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='constituency',
            name='country',
            field=models.ForeignKey(related_name='constituencies', blank=True, to='representatives.Country', null=True),
        ),
    ]



# coding: utf-8

import logging
import sys
from datetime import datetime

import django.dispatch
import ijson
import django
from django.apps import apps
from django.db import transaction
from django.utils import timezone
from django.utils.text import slugify

from representatives.models import (Address, Constituency, Country, Email,
                                    Group, Mandate, Phone, Representative,
                                    WebSite, Chamber)

logger = logging.getLogger(__name__)

representative_pre_import = django.dispatch.Signal(
    providing_args=['representative_data'])


def _parse_date(date):
    return datetime.strptime(date, "%Y-%m-%dT00:%H:00").date()


class GenericImporter(object):

    def pre_import(self):
        self.import_start_datetime = timezone.now()

    def post_import(self):
        # Clean not touched models
        models = [Representative, Group, Constituency,
                  Mandate, Address, Phone, Email, WebSite]
        for model in models:
            model.objects.filter(
                updated__lt=self.import_start_datetime).delete()

    def touch_model(self, model, **data):
        '''
        This method create or look up a model with the given data
        it saves the given model if it exists, updating its
        updated field
        '''
        instance, created = model.objects.get_or_create(**data)

        if not created:
            if instance.updated < self.import_start_datetime:
                instance.save()     # Updates updated field

        return (instance, created)


class ParltrackImporter(GenericImporter):
    url = 'http://parltrack.euwiki.org/dumps/ep_meps_current.json.xz'
    check_etag = True

    def parse_date(self, date):
        return _parse_date(date)

    def __init__(self):
        self.cache = {
            'countries': {c.name: c.pk for c in Country.objects.all()},
        }
        self.ep_chamber, _ = Chamber.objects.get_or_create(
            name='European Parliament', abbreviation='EP')
        self.ep_constituency, _ = Constituency.objects.get_or_create(
            name='European Parliament')
        self.ep_group, _ = Group.objects.get_or_create(
            name='European Parliament', kind='chamber', abbreviation='EP',
            chamber=self.ep_chamber)

    @transaction.atomic
    def manage_mep(self, mep_json):
        '''
        Import a mep as a representative from the json dict fetched from
        parltrack
        '''

        # Some versions of memopol will connect to this and skip inactive meps.
        responses = representative_pre_import.send(sender=self,
                representative_data=mep_json)

        for receiver, response in responses:
            if response is False:
                logger.debug(
                    'Skipping MEP %s', mep_json['Name']['full'])
                return

        changed = False
        slug = slugify('%s-%s' % (
            mep_json["Name"]["full"] if 'full' in mep_json["Name"]
            else mep_json["Name"]["sur"] + " " + mep_json["Name"]["family"],
            _parse_date(mep_json["Birth"]["date"])
        ))
        try:
            representative = Representative.objects.get(slug=slug)
        except Representative.DoesNotExist:
            representative = Representative(slug=slug)
            changed = True

        # Save representative attributes
        self.import_representative_details(representative, mep_json, changed)

        self.add_mandates(representative, mep_json)

        self.add_contacts(representative, mep_json)

        logger.debug('Imported MEP %s', unicode(representative))

        return representative

    def import_representative_details(self, representative, mep_json, changed):
        if representative.active != mep_json['active']:
            representative.active = mep_json['active']
            changed = True

        if mep_json.get("Birth"):
            birth_date = _parse_date(mep_json["Birth"]["date"])
            if representative.birth_date != birth_date:
                representative.birth_date = birth_date
                changed = True
            if "place" in mep_json["Birth"]:
                birth_place = mep_json["Birth"]["place"]
                if representative.birth_place != birth_place:
                    representative.birth_place = birth_place
                    changed = True

        if representative.first_name != mep_json["Name"]["sur"]:
            representative.first_name = mep_json["Name"]["sur"]
            changed = True

        last_name = mep_json["Name"]["family"]

        if representative.full_name != mep_json["Name"]["full"]:
            representative.full_name = mep_json["Name"]["full"]
            changed = True

        if representative.photo != mep_json["Photo"]:
            representative.photo = mep_json["Photo"]
            changed = True

        fix_last_name_with_prefix = {
            "Esther de LANGE": "de LANGE",
            "Patricia van der KAMMEN": "van der KAMMEN",
            "Judith A. MERKIES": "MERKIES",
            "Heinz K. BECKER": "BECKER",
            "Cornelis de JONG": "de JONG",
            "Peter van DALEN": "van DALEN",
            "Sophia in 't VELD": "in 't VELD",
            "Marielle de SARNEZ": "de SARNEZ",
            "Anne E. JENSEN": "JENSEN",
            "Wim van de CAMP": "van de CAMP",
            "Lambert van NISTELROOIJ": "van NISTELROOIJ",
            "Johannes Cornelis van BAALEN": "van BAALEN",
            "Ioannis A. TSOUKALAS": "TSOUKALAS",
            "Pilar del CASTILLO VERA": "del CASTILLO VERA",
            "Luis de GRANDES PASCUAL": "de GRANDES PASCUAL",
            "Philippe de VILLIERS": "de VILLIERS",
            "Daniël van der STOEP": "van der STOEP",
            "William (The Earl of) DARTMOUTH": "(The Earl of) Dartmouth",
            "Bairbre de BRÚN": u'de Br\xfan',
            "Karl von WOGAU": u'von WOGAU',
            "Ieke van den BURG": u'van den BURG',
            "Manuel António dos SANTOS": u'dos SANTOS',
            "Paul van BUITENEN": u'van BUITENEN',
            "Elly de GROEN-KOUWENHOVEN": u'de GROEN-KOUWENHOVEN',
            "Margrietus van den BERG": u'van den BERG',
            u'Dani\xebl van der STOEP': u'van der STOEP',
            "Alexander Graf LAMBSDORFF": u'Graf LAMBSDORFF',
            u'Bairbre de BR\xdaN': u'de BR\xdaN',
            'Luigi de MAGISTRIS': 'de MAGISTRIS',
        }

        if fix_last_name_with_prefix.get(representative.full_name):
            last_name = fix_last_name_with_prefix[representative.full_name]
        elif last_name == "J.A.J. STASSEN":
            last_name = "STASSEN"

        if representative.last_name != last_name:
            representative.last_name = last_name
            changed = True

        gender_convertion_dict = {u"F": 1, u"M": 2}
        if 'Gender' in mep_json:
            gender = gender_convertion_dict.get(mep_json['Gender'], 0)
        else:
            gender = 0
        if representative.gender != gender:
            representative.gender = gender
            changed = True

        cv = "\n".join([cv_title for cv_title in mep_json.get("CV", [])])
        if representative.cv != cv:
            representative.cv = cv
            changed = True

        if changed:
            representative.save()

    def add_mandates(self, representative, mep_json):
        def create_mandate(mandate_data, representative, group, constituency):
            if mandate_data.get("start"):
                begin_date = _parse_date(mandate_data.get("start"))
            if mandate_data.get("end"):
                end_date = _parse_date(mandate_data.get("end"))

            role = mandate_data['role'] if 'role' in mandate_data else ''
            mandate, _ = Mandate.objects.get_or_create(
                representative=representative,
                group=group,
                constituency=constituency,
                role=role,
                begin_date=begin_date,
                end_date=end_date
            )

            if _:
                logger.debug('Created mandate %s with %s', mandate.pk,
                             mandate_data)

        # Committee
        for mandate_data in mep_json.get('Committees', []):
            if mandate_data.get("committee_id"):
                group, _ = self.touch_model(model=Group,
                        abbreviation=mandate_data['committee_id'],
                        kind='committee', name=mandate_data['Organization'],
                        chamber=self.ep_chamber)

                create_mandate(mandate_data, representative, group,
                               self.ep_constituency)

        # Delegations
        for mandate_data in mep_json.get('Delegations', []):
            group, _ = self.touch_model(model=Group,
                                        kind='delegation',
                                        name=mandate_data['Organization'],
                                        chamber=self.ep_chamber
                                        )

            create_mandate(mandate_data, representative, group,
                           self.ep_constituency)

        # Group
        convert = {
            "S&D": "SD",
            "NA": "NI",
            "ID": "IND/DEM",
            "PPE": "EPP",
            "Verts/ALE": "Greens/EFA"}
        for mandate_data in mep_json.get('Groups', []):
            if not mandate_data.get('groupid'):
                continue

            if isinstance(mandate_data.get('groupid'), list):
                abbreviation = mandate_data.get('groupid')[0]
            else:
                abbreviation = mandate_data.get('groupid')

            abbreviation = convert.get(abbreviation, abbreviation)
            group, _ = self.touch_model(model=Group,
                                        abbreviation=abbreviation,
                                        kind='group',
                                        name=mandate_data['Organization'],
                                        chamber=self.ep_chamber
                                        )

            create_mandate(mandate_data, representative, group,
                           self.ep_constituency)

        # Countries
        for mandate_data in mep_json.get('Constituencies', []):
            if not mandate_data:
                continue

            _country = Country.objects.get(name=mandate_data['country'])

            group, _ = self.touch_model(model=Group,
                                        abbreviation=_country.code,
                                        kind='country',
                                        name=_country.name
                                        )

            local_party = mandate_data['party'] if mandate_data[
                'party'] and mandate_data['party'] != '-' else 'unknown'

            country_id = (self.cache['countries'].get(mandate_data['country'])
                if 'country' in mandate_data else None)

            save_constituency = False
            try:
                constituency = Constituency.objects.get(name=local_party)
            except Constituency.DoesNotExist:
                constituency = Constituency(name=local_party)
                save_constituency = True

            if constituency.country_id != country_id:
                constituency.country_id = country_id
                save_constituency = True

            if save_constituency:
                constituency.save()

            create_mandate(mandate_data, representative, group, constituency)

            create_mandate(mandate_data, representative, self.ep_group,
                           self.ep_constituency)

        # Organisations
        for mandate_data in mep_json.get('Staff', []):

            group, _ = self.touch_model(model=Group,
                                        abbreviation='',
                                        kind='organization',
                                        name=mandate_data['Organization']
                                        )

            create_mandate(mandate_data, representative, group,
                           self.ep_constituency)

    def add_contacts(self, representative, mep_json):
        # Addresses
        if mep_json.get('Addresses', None):
            address = mep_json.get('Addresses')

            belgium = Country.objects.get(name="Belgium")
            france = Country.objects.get(name="France")

            for city in address:
                if city in ['Brussels', 'Strasbourg']:
                    if city == 'Brussels':
                        country = belgium
                        street = u"rue Wiertz / Wiertzstraat"
                        number = '60'
                        postcode = '1047'
                        name = "Brussels European Parliament"
                    elif city == 'Strasbourg':
                        country = france
                        street = u"Av. du Président Robert Schuman - CS 91024"
                        number = '1'
                        postcode = '67070'
                        name = "Strasbourg European Parliament"

                    address_model, _ = self.touch_model(model=Address,
                        representative=representative, country=country,
                        city=city,
                        floor=address[city]['Address']['Office'][:3],
                        office_number=address[city]['Address']['Office'][3:],
                        street=street, number=number, postcode=postcode,
                        kind='official', name=name)

                    self.touch_model(model=Phone,
                        representative=representative, address=address_model,
                        kind='office phone',
                        number=address[city].get('Phone', ''))

        # Emails
        if mep_json.get('Mail', None):
            mails = mep_json.get('Mail')
            if not isinstance(mails, list):
                mails = list(mails)

            for mail in mails:
                self.touch_model(
                    model=Email,
                    representative=representative,
                    kind=('official' if '@europarl.europa.eu' in mail
                        else 'other'),
                    email=mail)

        # EP page
        changed = False
        try:
            site = WebSite.objects.get(kind='EP',
                                       representative=representative)
        except WebSite.DoesNotExist:
            site = WebSite(kind='EP', representative=representative)
            changed = True

        uid = mep_json['UserID']
        url = 'http://www.europarl.europa.eu/meps/en/%s/_home.html' % uid
        if site.url != url:
            site.url = url
            changed = True

        if changed:
            site.save()

        # WebSite
        websites = mep_json.get('Homepage', [])
        for url in websites:
            self.touch_model(model=WebSite,
                             url=url,
                             representative=representative
                             )

        if mep_json.get('Twitter', None):
            self.touch_model(model=WebSite,
                             representative=representative,
                             kind='twitter',
                             url=mep_json.get('Twitter')[0]
                             )

        if mep_json.get('Facebook', None):
            self.touch_model(model=WebSite,
                             representative=representative,
                             kind='facebook',
                             url=mep_json.get('Facebook')[0]
                             )


def main(stream=None):
    if not apps.ready:
        django.setup()

    importer = ParltrackImporter()
    GenericImporter.pre_import(importer)

    for data in ijson.items(stream or sys.stdin, 'item'):
        importer.manage_mep(data)
    # Commenting for now, it's a bit dangerous, if a json file was corrupt it
    # would drop valid data !
    # importer.post_import()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def migrate_constituencies(apps, schema_editor):
    """
    Re-save constituencies to recompute fingerprints
    """
    Constituency = apps.get_model("representatives", "Constituency")
    for c in Constituency.objects.all():
        c.save()


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0016_chamber_migrate_data'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mandate',
            name='role',
            field=models.CharField(default=b'', help_text=b'Eg.: president of a political group', max_length=255, blank=True),
        ),

        migrations.RunPython(migrate_constituencies)
    ]



# coding: utf-8

from datetime import datetime

from django.db import models
from django.utils.encoding import smart_unicode
from django.utils.functional import cached_property


class TimeStampedModel(models.Model):
    """
    An abstract base class model that provides self-updating
    ``created`` and ``modified`` fields.
    """

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class Country(models.Model):
    name = models.CharField(max_length=255)
    code = models.CharField(max_length=2, unique=True)

    def __unicode__(self):
        return u'{} [{}]'.format(self.name, self.code)


class Representative(TimeStampedModel):
    """
    Base model for representatives
    """

    slug = models.SlugField(max_length=100, unique=True)
    first_name = models.CharField(max_length=255, blank=True, default='')
    last_name = models.CharField(max_length=255, blank=True, default='')
    full_name = models.CharField(max_length=255)
    GENDER = (
        (0, "N/A"),
        (1, "F"),
        (2, "M"),
    )
    gender = models.SmallIntegerField(choices=GENDER, default=0)
    birth_place = models.CharField(max_length=255, blank=True, default='')
    birth_date = models.DateField(blank=True, null=True)
    cv = models.TextField(blank=True, default='')
    photo = models.CharField(max_length=512, null=True)
    active = models.BooleanField(default=False)

    def __unicode__(self):
        return smart_unicode(self.full_name)

    def gender_as_str(self):
        genders = {0: 'N/A', 1: 'F', 2: 'M'}
        return genders[self.gender]

    class Meta:
        ordering = ['last_name', 'first_name']

# Contact related models


class Contact(TimeStampedModel):
    representative = models.ForeignKey(Representative)

    class Meta:
        abstract = True


class Email(Contact):
    email = models.EmailField()
    kind = models.CharField(max_length=255, blank=True, default='')


class WebSite(Contact):
    url = models.CharField(max_length=2048, blank=True, default='')
    kind = models.CharField(max_length=255, blank=True, default='')


class Address(Contact):
    country = models.ForeignKey(Country)
    city = models.CharField(max_length=255, blank=True, default='')
    street = models.CharField(max_length=255, blank=True, default='')
    number = models.CharField(max_length=255, blank=True, default='')
    postcode = models.CharField(max_length=255, blank=True, default='')
    floor = models.CharField(max_length=255, blank=True, default='')
    office_number = models.CharField(max_length=255, blank=True, default='')
    kind = models.CharField(max_length=255, blank=True, default='')
    name = models.CharField(max_length=255, blank=True, default='')
    location = models.CharField(max_length=255, blank=True, default='')


class Phone(Contact):
    number = models.CharField(max_length=255, blank=True, default='')
    kind = models.CharField(max_length=255, blank=True, default='')
    address = models.ForeignKey(Address, null=True, related_name='phones')


class Chamber(models.Model):
    """
    A representative chamber
    """
    name = models.CharField(max_length=255)
    country = models.ForeignKey('Country', null=True, related_name='chambers')
    abbreviation = models.CharField(max_length=10, blank=True, default='',
        db_index=True)

    def __unicode__(self):
        return u'{} [{}]'.format(self.name, self.abbreviation)


class Group(TimeStampedModel):
    """
    An entity represented by a representative through a mandate
    """
    name = models.CharField(max_length=511, db_index=True)
    abbreviation = models.CharField(max_length=10, blank=True, default='',
        db_index=True)
    kind = models.CharField(max_length=255, db_index=True)
    chamber = models.ForeignKey(Chamber, null=True, related_name='groups')

    @cached_property
    def active(self):
        return self.mandates.filter(end_date__gte=datetime.now()).exists()

    def __unicode__(self):
        return unicode(self.name)

    class Meta:
        ordering = ('name',)


class Constituency(TimeStampedModel):
    """
    An authority for which a representative has a mandate
    """
    name = models.CharField(max_length=255)
    country = models.ForeignKey('Country', null=True, blank=True,
        related_name='constituencies')

    @cached_property
    def active(self):
        return self.mandates.filter(end_date__gte=datetime.now()).exists()

    def __unicode__(self):
        return unicode(self.name)


class MandateManager(models.Manager):
    """ This satisfies repr(Mandate) """
    def get_queryset(self):
        return super(
            MandateManager,
            self).get_queryset().select_related(
            'group',
            'constituency')


class Mandate(TimeStampedModel):

    objects = MandateManager()

    group = models.ForeignKey(Group, null=True, related_name='mandates')
    constituency = models.ForeignKey(
        Constituency, null=True, related_name='mandates')
    representative = models.ForeignKey(Representative, related_name='mandates')
    role = models.CharField(
        max_length=255,
        blank=True,
        default='',
        help_text="Eg.: president of a political group"
    )
    begin_date = models.DateField(blank=True, null=True)
    end_date = models.DateField(blank=True, null=True)
    link = models.URLField()

    @property
    def active(self):
        return self.end_date >= datetime.now().date()

    def __unicode__(self):
        t = u'Mandate : {representative},{role} {group} for {constituency}'
        return t.format(
            representative=self.representative,
            role=(
                u' {} of'.format(
                    self.role) if self.role else u''),
            constituency=self.constituency,
            group=self.group)

    class Meta:
        ordering = ('-end_date',)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0014_chamber'),
    ]

    operations = [
        migrations.AddField(
            model_name='chamber',
            name='abbreviation',
            field=models.CharField(default=b'', max_length=10, db_index=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0002_fixtures'),
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('city', models.CharField(default=b'', max_length=255, blank=True)),
                ('street', models.CharField(default=b'', max_length=255, blank=True)),
                ('number', models.CharField(default=b'', max_length=255, blank=True)),
                ('postcode', models.CharField(default=b'', max_length=255, blank=True)),
                ('floor', models.CharField(default=b'', max_length=255, blank=True)),
                ('office_number', models.CharField(default=b'', max_length=255, blank=True)),
                ('kind', models.CharField(default=b'', max_length=255, blank=True)),
                ('name', models.CharField(default=b'', max_length=255, blank=True)),
                ('location', models.CharField(default=b'', max_length=255, blank=True)),
                ('country', models.ForeignKey(to='representatives.Country')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Constituency',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('fingerprint', models.CharField(unique=True, max_length=40)),
                ('name', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('email', models.EmailField(max_length=254)),
                ('kind', models.CharField(default=b'', max_length=255, blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('fingerprint', models.CharField(unique=True, max_length=40)),
                ('name', models.CharField(max_length=255)),
                ('abbreviation', models.CharField(default=b'', max_length=10, blank=True)),
                ('kind', models.CharField(default=b'', max_length=255, blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Mandate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('fingerprint', models.CharField(unique=True, max_length=40)),
                ('role', models.CharField(default=b'', help_text=b'Eg.: president of a political group at the European Parliament', max_length=25, blank=True)),
                ('begin_date', models.DateField(null=True, blank=True)),
                ('end_date', models.DateField(null=True, blank=True)),
                ('link', models.URLField()),
                ('constituency', models.ForeignKey(related_name='mandates', to='representatives.Constituency', null=True)),
                ('group', models.ForeignKey(related_name='mandates', to='representatives.Group', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Phone',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('number', models.CharField(default=b'', max_length=255, blank=True)),
                ('kind', models.CharField(default=b'', max_length=255, blank=True)),
                ('address', models.ForeignKey(related_name='phones', to='representatives.Address', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Representative',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('fingerprint', models.CharField(unique=True, max_length=40)),
                ('slug', models.SlugField(max_length=100)),
                ('remote_id', models.CharField(unique=True, max_length=255)),
                ('first_name', models.CharField(default=b'', max_length=255, blank=True)),
                ('last_name', models.CharField(default=b'', max_length=255, blank=True)),
                ('full_name', models.CharField(max_length=255)),
                ('gender', models.SmallIntegerField(default=0, choices=[(0, b'N/A'), (1, b'F'), (2, b'M')])),
                ('birth_place', models.CharField(default=b'', max_length=255, blank=True)),
                ('birth_date', models.DateField(null=True, blank=True)),
                ('cv', models.TextField(default=b'', blank=True)),
                ('photo', models.CharField(max_length=512, null=True)),
                ('active', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='WebSite',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('url', models.CharField(default=b'', max_length=2048, blank=True)),
                ('kind', models.CharField(default=b'', max_length=255, blank=True)),
                ('representative', models.ForeignKey(to='representatives.Representative')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='phone',
            name='representative',
            field=models.ForeignKey(to='representatives.Representative'),
        ),
        migrations.AddField(
            model_name='mandate',
            name='representative',
            field=models.ForeignKey(related_name='mandates', to='representatives.Representative'),
        ),
        migrations.AddField(
            model_name='email',
            name='representative',
            field=models.ForeignKey(to='representatives.Representative'),
        ),
        migrations.AddField(
            model_name='address',
            name='representative',
            field=models.ForeignKey(to='representatives.Representative'),
        ),
    ]



# coding: utf-8


class DelegationHelper:
    '''
    Helper class for building committees/delegations from rep json data
    given dicts for equivalences and abbreviations
    '''

    def __init__(self, equivs, abbrevs, committees=True):
        self.equivs = equivs
        self.abbrevs = abbrevs
        self.committees = committees

    def __call__(self, data):
        items = []
        start = data['mandat_debut']
        end = data.get('mandat_fin', None)

        if self.committees:
            gdata = (i['responsabilite'] for i in data['responsabilites'])
        else:
            gdata = ([i['responsabilite'] for i in data['responsabilites']] +
                [j['responsabilite'] for j in data['groupes_parlementaires']])

        for g in gdata:
            orga = g['organisme']
            role = g['fonction']

            if self.committees != orga.lower().startswith('commission'):
                continue

            if orga in self.equivs:
                orga = self.equivs[orga]

            item = {
                'abbr': self.abbrevs[orga] if orga in self.abbrevs else '',
                'name': orga,
                'role': role,
                'start': start
            }

            if end:
                item['end'] = end

            items.append(item)

        return items


def _get_rep_district_name(data):
    num = data.get('num_circo')
    nom = data.get('nom_circo')

    if num == 'nd':
        return nom
    else:
        ordinal = u'ère' if num == 1 else u'ème'
        return '%s (%d%s circonscription)' % (nom, num, ordinal)


_get_sen_committees = DelegationHelper({
    u"COMMISSION DES AFFAIRES EUROPÉENNES Commission des affaires européennes":
        u"Commission des affaires européennes",
    u"Commission de l'aménagement du territoire et du développement durable":
        (u"Commission du développement durable, des infrastructures, de "
            u"l'équipement et de l'aménagement du territoire")
}, {
    u"Commission de la culture, de l'éducation et de la communication":
        "SenComCult",
    u"Commission des affaires économiques": "SenComEco",
    u"Commission des affaires étrangères, de la défense et des forces armées":
        "SenComDef",
    u"Commission des affaires européennes": "SenComEU",
    u"Commission des affaires sociales": "SenComSoc",
    (u"Commission des finances, du contrôle budgétaire et des comptes "
        u"économiques de la nation"): "SenComFin",
    (u"Commission des lois constitutionnelles, de législation, du suffrage "
        u"universel, du Règlement et d'administration générale"): "SenComLois",
    (u"Commission du développement durable, des infrastructures, de "
        u"l'équipement et de l'aménagement du territoire"): "SenComDevD",
    u"Commission sénatoriale pour le contrôle de l'application des lois":
        "SenComAppL"
})

_get_an_committees = DelegationHelper({}, {
    (u"Commission chargée de l'application de l'article 26 de la "
        u"constitution"): "AnComImmu",
    u"Commission de la défense nationale et des forces armées": "AnComDef",
    u"Commission des affaires culturelles et de l'éducation": "AnComCult",
    u"Commission des affaires économiques": "AnComEco",
    u"Commission des affaires étrangères": "AnComEtrg",
    u"Commission des affaires européennes": "AnComEU",
    u"Commission des affaires sociales": "AnComSoc",
    (u"Commission des finances, de l'économie générale et du contrôle "
        u"budgétaire"): "AnComFin",
    (u"Commission des lois constitutionnelles, de la législation et de "
        u"l'administration générale de la république"): "AnComLois",
    u"Commission du développement durable et de l'aménagement du territoire":
        "AnComDevD"
})

_get_sen_delegations = DelegationHelper({}, {}, False)

_get_an_delegations = DelegationHelper({}, {}, False)


#
# Variant configuration
# - mail_domain is used to distinguish official vs personal emails
# - off_* fields are used for the official address of meps
# - mandates defines how mandates are created from the rep json
#
# Mandates are defined as follows
# - 'kind' indicates the group kind, a constant string
# - 'chamber' tells whether the group belongs to the chamber
# - 'from', if present, must be a function that takes the rep json and returns
#   an array of dicts; one group will be created from each item in the dict.
#   When 'from' is not present, only one group wil be created using the rep
#   json (IOW, 'from' defaults to lambda repjson: [repjson])
# - 'name', 'abbr', 'role', 'start', 'end' are strings that are interpolated
#   against the rep json or items returned by 'from'.
# - 'name_path', 'abbr_path', etc. can replace 'name', 'abbr'... by specifying
#   a slash-separated dictionnary path where the value is to be found in the
#   rep json or item returned by 'from'
# - 'name_fn', 'abbr_fn', etc. can also replace 'name', 'abbr'... by a
#   function that takes the input item (rep json or item returned by 'from')
#   and returns the value
#
FranceDataVariants = {
    "AN": {
        "chamber": u"Assemblée nationale",
        "abbreviation": "AN",
        "chamber_url_field": "url_an",
        "mail_domain": "@assemblee-nationale.fr",
        "off_city": "Paris",
        "off_street": u"Rue de l'Université",
        "off_number": "126",
        "off_code": "75355",
        "off_name": u"Assemblée nationale",
        "mandates": [
            {
                "kind": "chamber",
                "chamber": True,
                "abbr": "AN",
                "name": u"Assemblée nationale",
                "role": u"Député",
                "start": "%(mandat_debut)s",
                "end": "%(mandat_fin)s"
            },
            {
                "kind": "group",
                "chamber": True,
                "abbr": "%(groupe_sigle)s",
                "name_path": "groupe/organisme",
                "role_path": "groupe/fonction",
                "start": "%(mandat_debut)s",
                "end": "%(mandat_fin)s"
            },
            {
                "kind": "department",
                "abbr": "%(num_deptmt)s",
                "name": "%(nom_circo)s",
                "start": "%(mandat_debut)s",
                "end": "%(mandat_fin)s"
            },
            {
                "kind": "district",
                "abbr": "%(num_deptmt)s-%(num_circo)s",
                "name_fn": _get_rep_district_name,
                "start": "%(mandat_debut)s",
                "end": "%(mandat_fin)s"
            },
            {
                "kind": "committee",
                "chamber": True,
                "from": _get_an_committees,
                "abbr": "%(abbr)s",
                "name": "%(name)s",
                "role": "%(role)s",
                "start": "%(start)s",
                "end": "%(end)s"
            },
            {
                "kind": "delegation",
                "chamber": True,
                "from": _get_an_delegations,
                "abbr": "%(abbr)s",
                "name": "%(name)s",
                "role": "%(role)s",
                "start": "%(start)s",
                "end": "%(end)s"
            }
        ]
    },

    "SEN": {
        "chamber": u"Sénat",
        "abbreviation": "SEN",
        "chamber_url_field": "url_institution",
        "mail_domain": "@senat.fr",
        "off_city": "Paris",
        "off_street": u"Rue de Vaugirard",
        "off_number": "15",
        "off_code": "75291",
        "off_name": u"Palais du Luxembourg",
        "mandates": [
            {
                "kind": "chamber",
                "chamber": True,
                "abbr": "SEN",
                "name": u"Sénat",
                "role": u"Sénateur",
                "start": "%(mandat_debut)s",
                "end": "%(mandat_fin)s"
            },
            {
                "kind": "group",
                "chamber": True,
                "abbr": "%(groupe_sigle)s",
                "name_path": "groupe/organisme",
                "role_path": "groupe/fonction",
                "start": "%(mandat_debut)s",
                "end": "%(mandat_fin)s"
            },
            {
                "kind": "department",
                "abbr": "%(num_deptmt)s",
                "name": "%(nom_circo)s",
                "start": "%(mandat_debut)s",
                "end": "%(mandat_fin)s"
            },
            {
                "kind": "district",
                "abbr": "%(num_deptmt)s-%(num_circo)s",
                "name_fn": _get_rep_district_name,
                "start": "%(mandat_debut)s",
                "end": "%(mandat_fin)s"
            },
            {
                "kind": "committee",
                "chamber": True,
                "from": _get_sen_committees,
                "abbr": "%(abbr)s",
                "name": "%(name)s",
                "role": "%(role)s",
                "start": "%(start)s",
                "end": "%(end)s"
            },
            {
                "kind": "delegation",
                "chamber": True,
                "from": _get_sen_delegations,
                "abbr": "%(abbr)s",
                "name": "%(name)s",
                "role": "%(role)s",
                "start": "%(start)s",
                "end": "%(end)s"
            }
        ]
    }
}



from django.core.management.base import BaseCommand


class RemoveCommand(BaseCommand):
    conditions = {}
    manager = None

    def add_arguments(self, parser):
        parser.add_argument('--dry',
            action='store_true',
            default=False,
            help='Do not actually execute delete')

    def handle(self, *args, **options):
        remove = self.manager.filter(**self.conditions)
        keep = self.manager.exclude(**self.conditions)

        print 'Delete %s %s, should remain %s' % (remove.count(),
            self.manager.model._meta.verbose_name_plural, keep.count())

        if options.get('dry', False):
            return

        remove.delete()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0008_constituency_country'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='mandate',
            options={'ordering': ('end_date',)},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0004_auto_20150709_1601'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mandate',
            name='role',
            field=models.CharField(default=b'', help_text=b'Eg.: president of a political group', max_length=25, blank=True),
        ),
    ]



import pytest
import os
import copy

from django.core.serializers.json import Deserializer
from representatives.models import Representative
from representatives.contrib.parltrack import import_representatives


@pytest.mark.django_db
def test_parltrack_import_representatives():
    fixture = os.path.join(os.path.dirname(__file__),
            'representatives_fixture.json')
    expected = os.path.join(os.path.dirname(__file__),
            'representatives_expected.json')

    # Disable django auto fields
    exclude = ('id', '_state', 'created', 'updated', 'fingerprint')

    with open(fixture, 'r') as f:
        import_representatives.main(f)

    missing = []
    with open(expected, 'r') as f:
        for obj in Deserializer(f.read()):
            compare = copy.copy(obj.object.__dict__)

            for field in exclude:
                if field in compare:
                    compare.pop(field)

            try:
                type(obj.object).objects.get(**compare)
            except:
                missing.append(compare)

    assert len(missing) is 0
    assert Representative.objects.count() == 2



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0003_auto_20150702_1827'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='representative',
            options={'ordering': ['last_name', 'first_name']},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime

from django.db import connection, migrations, models


def update_slugs(apps, schema_editor):
    """
    Include birthdate in slugs
    """

    # Get model managers
    Representative = apps.get_model("representatives", "Representative")

    for rep in Representative.objects.all():
        rep.slug = '%s-%s' % (rep.slug, rep.birth_date)
        rep.save()


def create_parl_websites(apps, schema_editor):
    """
    Prepare for remote_id removal by creating WebSite entities from it.
    """

    # Get model managers
    Representative = apps.get_model("representatives", "Representative")
    WebSite = apps.get_model("representatives", "WebSite")

    today = datetime.date.today()

    # EP
    ep_url = 'http://www.europarl.europa.eu/meps/en/%s/_home.html'
    qs = Representative.objects.filter(
        models.Q(mandates__end_date__gte=today) |
        models.Q(mandates__end_date__isnull=True),
        mandates__group__chamber__abbreviation='EP'
    )
    for rep in qs:
        changed = False
        url = ep_url % rep.remote_id

        try:
            site = WebSite.objects.get(representative=rep, kind='EP')
        except WebSite.DoesNotExist:
            site = WebSite(representative=rep, kind='EP', url=url)
            changed = True

        if site.url != url:
            site.url = url
            changed = True

        if changed:
            site.save()

    # AN/SEN
    for chamber in ['AN', 'SEN']:
        qs = Representative.objects.filter(
            models.Q(mandates__end_date__gte=today) |
            models.Q(mandates__end_date__isnull=True),
            mandates__group__chamber__abbreviation=chamber
        )
        for rep in qs:
            changed = False
            url = rep.remote_id

            try:
                site = WebSite.objects.get(representative=rep, kind=chamber)
            except WebSite.DoesNotExist:
                site = WebSite(representative=rep, kind=chamber, url=url)
                changed = True

            if site.url != url:
                site.url = url
                changed = True

            if changed:
                site.save()


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0019_remove_fingerprints'),
    ]

    operations = [
        migrations.RunPython(update_slugs),

        migrations.RunPython(create_parl_websites),

        migrations.RemoveField(
            model_name='representative',
            name='remote_id',
        ),

        migrations.AlterField(
            model_name='representative',
            name='slug',
            field=models.SlugField(unique=True, max_length=100),
        ),
    ]
